using System;
using System.Text;
using System.Xml;
using System.Xml.Schema;
using System.IO;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;

using KRW.Utility;
using EnterpriseDT.Net.Ftp;

namespace EventCal
{
	class EventCal
	{
		private bool _upload = true;
		private Parms _parms = null;
		private string calFile = String.Empty;

		private Dictionary<string, string> libs = new Dictionary<string, string>();

		[STAThread]
		static void Main(string[] args)
		{
			EventCal me = new EventCal();
			if (args.Length < 1) {
				me.showUsage();
				return;
			}

			// load the command line parms
			CmdLineParser cmdline = new CmdLineParser();
			cmdline.LoadCmdLine(args, false);
			if (cmdline.Is("no")) me._upload = false;

			me.calFile = cmdline.getByPos(1);
			if (!File.Exists(me.calFile)) {
				Console.WriteLine("Error: The calendar file '{0}' does not exist.", me.calFile);
				return;
			}

			me._parms = new Parms();
			me._parms.LoadParms(me.calFile);
			//me._parms.Dump();

			if (!me._parms.Is("events")) {
				Console.WriteLine("Error: No 'events' parameter specified. Stopping...");
				return;
			}

			// load any library stuff
			if (!me.loadLibrary()) {
				Console.WriteLine("Error: Library load failed.");
				return;
			}

			// load the events into the collection
			string outf = me.process();
			if (me._upload) {
				if (!String.IsNullOrEmpty(outf)) {
					string srvr = String.Empty;
					string user = String.Empty;
					string pass = String.Empty;
					string dir = String.Empty;
					string file = "event.txt";

					// we have a file to upload, make sure we have enough information to do the upload
					if (me._parms.Is("ftp")) srvr = me._parms.Get("ftp");
					if (me._parms.Is("ftpuser")) user = me._parms.Get("ftpuser");
					if (me._parms.Is("ftppass")) pass = me._parms.Get("ftppass");
					if (me._parms.Is("ftpdir")) dir = me._parms.Get("ftpdir");
					if (me._parms.Is("ftpfile")) file = me._parms.Get("ftpfile");
					if (cmdline.Is("pass")) pass = cmdline.Get("pass");

					// verify that we have all the information
					bool good = true;
					if (String.IsNullOrEmpty(srvr)) {
						Console.WriteLine("Error: No FTP server is defined.");
						good = false;
					}
					if (String.IsNullOrEmpty(user)) {
						Console.WriteLine("Error: No FTP user is defined.");
						good = false;
					}
					if (String.IsNullOrEmpty(pass)) {
						Console.WriteLine("Error: No FTP password is provided for the user.");
						good = false;
					}
					if (String.IsNullOrEmpty(file)) {
						Console.WriteLine("Error: No file name for the uploaded file is provided.");
						good = false;
					}

					// upload the result of the event calendar generation
					if (good) {
						if (!me.upload(outf, srvr, user, pass, dir, file)) {
							Console.WriteLine("Upload of '{0}' failed.", outf);
						}
					}
				}
			}
			else {
				Console.WriteLine("\nNOTICE: Due to -no command-line parm, the events will not be uploaded.");
			}

			return;
		}

		private bool loadLibrary()
		{
			if (_parms.Is("library")) {
				if (!File.Exists(_parms.Get("library"))) {
					Console.WriteLine("Error: The library file '{0}' does not exist.", _parms.Get("library"));
					return false;
				}

				awkLite awk = new awkLite(_parms.Get("library"), '=');
				awkRow row = new awkRow();

				while (awk.Next(row) > 0) {
					// skip comments
					if (row.isBlank) continue;
					if (row.line.StartsWith("*")) continue;
					if (row.flds > 1) {
						libs[row.nodes[0].Trim().ToUpper()] = row.glom(2).Trim();
					}
				}
				awk.Close();
			}

			return true;
		}

		// builds a new event file and returns the name of that file, returns empty string if 
		// no file was created or error, etc.
		private string process()
		{
			string yr = String.Empty;
			string mon = String.Empty;
			string key = String.Empty;
			bool start = false;
			string infile = String.Empty;
			string outfile = String.Empty;
			SortedDictionary<string, string> events = new SortedDictionary<string, string>();
			StreamWriter wtr = null;

			DateTime now = DateTime.Now;

			if (_parms.Is("events")) {
				infile = _parms.Get("events") + ".in";
				outfile = String.Format(@"output\{0}_{1}.txt", _parms.Get("events"), now.ToString("yyyyMMddHmm"));
				wtr = new StreamWriter(outfile);
			}

			awkLite awk = new awkLite(infile, "|");
			awkRow row = new awkRow();

			while (awk.Next(row) > 0) {
				if (row.isBlank) continue;
				if (row.line.StartsWith("*")) continue;		// skip comments
			   //Console.WriteLine(String.Format("Dump row: {0}", row.line));
				if (row.line.ToUpper().Trim() == "[BEGINDATA]") {
					wtr.WriteLine(row.line.ToUpper().Trim());
					start = true;
				}
				else if (row.line.ToUpper().Trim() == "[ENDDATA]") {
					dumpEvents(ref events, wtr);
					wtr.WriteLine(row.line.ToUpper().Trim());
					start = false;
				}
				else {
					if (start) {
						// main processing
						switch (row.nodes[0].ToUpper()) {
						case "YEAR":
							yr = row.nodes[1];
							if (yr.Length < 4) yr = "20" + yr;
							break;
						case "MONTH":
							mon = row.nodes[1];
							if (mon.Length < 2) mon = "0" + mon;
							break;
						default:
							// format should be day:value
							if (row.flds > 1) {
								key = String.Format("{0}{1}{2}", yr, mon, (row.nodes[0].Length < 2 ? "0" + row.nodes[0] : row.nodes[0]));
								events[key] = nodeValue(row.glom(2).Trim());
							}
							break;
						}
					}
					else {
						wtr.WriteLine(row.line);
					}
				}
			}
			awk.Close();
			wtr.Close();

			return outfile;
		}

		private void dumpEvents(ref SortedDictionary<string, string> events, StreamWriter wtr)
		{
			//Console.WriteLine("Dumping events..." + events.Count);
			foreach (KeyValuePair<string, string> kvp in events) {
				wtr.WriteLine("{0}|{1}", kvp.Key, kvp.Value);
			}
		}

		private string nodeValue(string val)
		{
			if (val.StartsWith("@")) {
				string key = val.Substring(1).ToUpper();
				// it may be a variable
				if (libs.ContainsKey(key)) {
					return libs[key];
				}
			}

			return val;
		}

		protected bool upload(string inFn, string srvr, string user, string pass, string dir, string outFn)
		{
			bool rc = true;
			FTPConnection ftp = new FTPConnection();
			try {
				ftp.ServerAddress = srvr;
				ftp.UserName = user;
				ftp.Password = pass;

				ftp.Connect();

				if (!String.IsNullOrEmpty(dir)) {
					ftp.ChangeWorkingDirectory(dir);
				}
				Console.WriteLine("Getting ready to upload...");
				ftp.UploadFile(inFn, outFn);
				Console.WriteLine("Finished.");
			} 
			catch (Exception ex) {
				Console.WriteLine("Error: Exception occurred. {0}", ex.Message);
				rc = false;
			}
			finally {
				ftp.Close();
			}

			return rc;
		}

		protected void showUsage()
		{
			Console.WriteLine("\neventcal calendar-config-file [-no] [-pass=password]\n");
			Console.WriteLine("\t -no = no upload of completed file");
			Console.WriteLine("\t -pass = optional password for FTP site, if not in .cal file");
		}
	}
}
