using System;
using System.Text;
using System.IO;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Security.Cryptography;

/// <summary>
/// 
/// This file is a set of utility classes that are useful in many programs.
///
/// Classes:
/// 	Parms
///   CmdLineParser
///   StaticMethods
///      - Guid()
///      - MakeSHA1()
///   awkLite, awkRow
///   Dtf (depends on awkLite, awkRow)
/// 	
/// </summary>

namespace KRW.Utility {

// ========================================================
//   Parms:  load parameters from an input file
// ========================================================
public class Parms
{
	protected Dictionary<string, string> _parms = new Dictionary<string, string>();
	
	public int LoadParms(string filename)
	{
		StreamReader input = new StreamReader(filename);
		string buffer = input.ReadLine();
		string[] split = {};
		while (buffer != null) {
			split = buffer.Split('=');
			if (split.Length > 1) {
				_parms[split[0].ToUpper().Trim()] = split[1].Trim();
			}
			buffer = input.ReadLine();
		}
		input.Close();
		return _parms.Count;
	}

	public string Get(string key)
	{
		if (_parms.ContainsKey(key.ToUpper())) {
			return _parms[key.ToUpper()];
		}
		return "";
	}

	public int GetInt(string key)
	{
		if (_parms.ContainsKey(key.ToUpper())) {
			return Convert.ToInt32(_parms[key.ToUpper()]);
		}
		return -1;
	}

   public bool Is(string key)
   {
		if (_parms.ContainsKey(key.ToUpper())) return true;
      return false;
   }

   public bool isTrue(string key)
   {
      if (_parms.ContainsKey(key.ToUpper())) {
         string x = _parms[key.ToUpper()].ToUpper();
         if (x == "TRUE" || x == "1") return true;
      }
      return false;
   }

   public void Set(string key, string value)
   {
      _parms[key.ToUpper()] = value;
   }

   public void Dump()
   {
      foreach (KeyValuePair<string, string> kvp in _parms) {
         Console.WriteLine("Parm: {0} = {1}", kvp.Key, kvp.Value);
      }
   }
}

public class CmdLineParser
{
	protected Dictionary<string, string> _key = new Dictionary<string, string>();
	protected List<string> _position = new List<string>();
   protected bool _caseSensitive = false;
	
	public void LoadCmdLine(string[] args, bool caseSensitive)
	{
      _caseSensitive = caseSensitive;

      string[] split = {};
      string current = "";
      bool keybased = false;
      for (int i=0; i < args.Length; i++)
      {
         keybased = false;
         current = args[i];

         // is key-based
         if (args[i].StartsWith("--")) {
            keybased = true;
            current = args[i].Substring(2);
         }
         else if (args[i].StartsWith("-")) {
            keybased = true;
            current = args[i].Substring(1);
         }
         else if (args[i].StartsWith("/")) {
            keybased = true;
            current = args[i].Substring(1);
         }
         if (keybased) {
            split = current.Split('=');
            if (split.Length > 1) {
               _key[Normalize(split[0])] = split[1];
            }
            else {
               _key[Normalize(split[0])] = "";
            }
         }
         else {
            // position based, don't normalize it
            _position.Add(current);
         }
      }
	}

   private string Normalize(string inp)
   {
      if (!_caseSensitive) return inp.ToUpper();
      return inp;
   }

   public bool Is(string key)
   {
      if (_key.ContainsKey(Normalize(key))) return true;
      return false;
   }

   public string Get(string key)
   {
      if (_key.ContainsKey(Normalize(key))) return _key[Normalize(key)];
      return "";
   }

   public string getByPos(int pos)
   {
      if (_position.Count >= pos) return _position[pos-1];
      return "";
   }
}

public class StaticMethods
{
   static public string MakeSHA1(string clear) {
      StringBuilder sBuilder = new StringBuilder();
      byte[] key = System.Text.Encoding.ASCII.GetBytes(clear);
      byte[] hash = SHA1.Create().ComputeHash(key);
      for (int i = 0; i < hash.Length; i++) {
        sBuilder.Append(hash[i].ToString("x2"));
      }
      return sBuilder.ToString();
   }

   static public string Guid(int format) {
      Guid g = System.Guid.NewGuid();
      switch (format) {
      case 0:
      default:
         return g.ToString();
      case 1:
         return g.ToString("N");
      }
   }
}

// ========================================================
//  awkLite (& awkRow): support awk-like input processing
// ========================================================
public class awkLite
{
	protected StreamReader _input = null;
	protected awkRow _row = new awkRow();
	protected Char[] _delim = new Char[] {' '};
	protected int _lineCount = 0;
   protected string _strDelim = null;
	
	public awkLite(string filename)
	{
		// flddelimiter defaults to whitespace
		_input = new StreamReader(filename);
		_delim[0] = ' ';
	}

	public awkLite(string filename, Char fldDelimiter)
	{
		_input = new StreamReader(filename);
		_delim[0] = fldDelimiter;
	}

	public awkLite(string filename, Char fldDelimiter, System.Text.Encoding encoding)
	{
		_input = new StreamReader(filename, encoding);
		_delim[0] = fldDelimiter;
	}

   public awkLite(string filename, string delimiter)
   {
      // allow splitting of row by a string
		_input = new StreamReader(filename);
		_strDelim = delimiter;
   }

	public int Next(awkRow row)
	{
		if (_input == null) return -1;

		string buffer = _input.ReadLine();
		if (buffer == null) return -1;

		// we actually have a row
		_lineCount++;
      if (_strDelim == null) {
         // delimiter is a single character
         return row.row(buffer, _delim);
      }

      // delimiter is a string
      return row.row(buffer, _strDelim);
	}

	public void Close()
	{
		if (_input == null) return;
		_input.Close();
		_input = null;
	}

	public int LineCount()
	{
		return _lineCount;
	}
}

public class awkRaw : awkRow
{
   public string raw = "";

   public override int row(string buffer, string delimiter)
   {
      raw = buffer;
      return base.row(buffer, delimiter);
   }
}

public class awkRow
{
	public string line = "";
	public string[] nodes = {};
	public int flds = 0;
	public bool isBlank = true;
	protected string[] delim = {""};

	public int row(string buffer, Char[] delimiter)
	{
      return row(buffer, new string(delimiter));
	}

   public string get(int nodeSub)
   {
      if (flds < nodeSub) return "";
      return nodes[nodeSub - 1].Trim();
   }

   public string getHtml(int nodeSub)
   {
      if (flds < nodeSub) return "&nbsp;";
      if (nodes[nodeSub-1].Trim().Length < 1) return "&nbsp;";
      return nodes[nodeSub - 1].Trim();
   }

	public virtual int row(string buffer, string delimiter)
	{
      delim[0] = delimiter;
		line = buffer.Trim();
		nodes = line.Split(delim, 255, System.StringSplitOptions.None);
		flds = nodes.Length;
		isBlank = false;
		if (flds < 1 || line.Length < 1) isBlank = true;
		return flds;
	}

	public string glom(int startNode)
	{
		return glom(startNode, flds-startNode+1 , delim[0]);
	}

	public string glom(int startNode, int len)
	{
		return glom(startNode, len, delim[0]);
	}

	public string glom(int startNode, int len, string delimiter) 
	{
		int cnt = 0;
		string strg = "";
		if (startNode <= flds) {
			for (int i=startNode-1; cnt < len && i < flds; i++) {
				//if (strg.Length > 0) strg.Append(delimiter);
				if (strg.Length > 0) strg += delimiter;
				strg += nodes[i];
			}
		}

		return strg;
	}

	public void copy(awkRow src)
	{
		 row(src.line, src.delim[0]);
	}
}

public class Dtf
{
   protected awkLite _awk = null;
   protected awkRow _row = new awkRow();
   protected bool _bHdrs = false;
   protected bool _bRowAvailable = false;
   protected string _empty = "";

	protected Dictionary<string, int> _colSubs = new Dictionary<string, int>();

   public Dtf(string filename, Char delimiter, bool bIncludesHdrs)
   {
      _awk = new awkLite(filename, delimiter);
      if (bIncludesHdrs) _initHdrs();
   }

   public Dtf(string filename, Char delimiter, bool bIncludesHdrs, System.Text.Encoding encoding)
   {
      _awk = new awkLite(filename, delimiter, encoding);
      if (bIncludesHdrs) _initHdrs();
   }

   private void _initHdrs()
   {
      _bHdrs = true;
      // we must process the headers line
      if (Next()) {
         for (int i=0; i < _row.flds; i++) {
            _colSubs[_row.nodes[i].ToUpper()] = i+1;
         }
      }
   }

   // Next():  will retrieve the next row and prepare for use
   public bool Next()
   {
      _bRowAvailable = false;
      if (_awk != null) {
         // skip blank rows
         while (_awk.Next(_row) >= 0) {
             if (!_row.isBlank) {
                _bRowAvailable = true;
                break;
             }
         }
      }

      return _bRowAvailable;
   }

   public void Close()
   {
      if (_awk != null) _awk.Close();
      _awk = null;
   }

   public string getString(int col)
   {
      if (!_bRowAvailable) return _empty;
      if (_row.isBlank || _row.flds < col || col < 1) return _empty;
      return _row.nodes[col-1];
   }

   public int getInt(int col)
   {
      if (!_bRowAvailable) return 0;
      if (_row.isBlank || _row.flds < col || col < 1) return 0;
      try {
         return Convert.ToInt32(_row.nodes[col-1]);
      } 
      catch {
         // if we can convert, then return 0
         return 0;
      }
   }

   // can retrieve values by column name also, 
   // note: this will be slower, it is better to cache the column subscripts 
   //       and retrieve by int rather than string
   public string getString(string hdr) { return getString(Col(hdr)); }
   public int getInt(string hdr) { return getInt(Col(hdr)); }

   // Col: returns the column subscript that matches the hdr passed in,
   //       if anything fails, returns -1
   public int Col(string hdr)
   {
      if (!_bHdrs) return -1;
      if (_colSubs.ContainsKey(hdr.ToUpper())) return _colSubs[hdr.ToUpper()];
      return -1;
   }

   public int numFields()
   {
      return _row.flds;
   }
}

}	// end of the namespace KRW.Utility
